# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import logging
import sys
from importlib import import_module
from pathlib import Path

from ..constants import DEFAULT_PROVIDER, PROVIDERS


def build_command(parser, args):
    logging.basicConfig(level=logging.INFO, format="[%(levelname)s] %(message)s")

    input_provider = import_module(f".{args.input_format}", "cici.providers")
    output_provider = import_module(f".{args.output_format}", "cici.providers")

    if not args.filename:
        args.filename = input_provider.CI_FILE

    if not Path(args.filename).exists():
        parser.error(f"file not found: {args.filename}")

    file = input_provider.load(args.filename)
    output_provider.dump(file, sys.stdout)


def build_parser(subparsers):
    parser = subparsers.add_parser(
        "build", help="build BuildGarden CI file into target format"
    )
    parser.add_argument("filename", nargs="?")
    parser.add_argument(
        "-f",
        "--from",
        dest="input_format",
        choices=PROVIDERS,
        default=DEFAULT_PROVIDER,
        help=f"input format [{DEFAULT_PROVIDER}]",
    )
    parser.add_argument(
        "-t",
        "--to",
        dest="output_format",
        choices=PROVIDERS,
        default=DEFAULT_PROVIDER,
        help=f"output format [{DEFAULT_PROVIDER}]",
    )
    parser.add_argument(
        "-o",
        "--output",
        metavar="DIR",
        dest="output_path",
        type=Path,
        default=Path.cwd().absolute(),
    )
    parser.set_defaults(func=build_command)
    return parser



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from pathlib import Path

BASE_DIR = Path(__file__).parent.absolute()

SCHEMA_DIR = BASE_DIR / "schema"

COMMAND_DIR = BASE_DIR / "cli"

COMMANDS = sorted(
    [path.stem for path in COMMAND_DIR.glob("*.py") if not path.stem.startswith("_")]
)

PROVIDER_DIR = BASE_DIR / "providers"

PROVIDERS = sorted(
    [path.stem for path in PROVIDER_DIR.glob("*.py") if not path.stem.startswith("_")]
    + [
        path.stem
        for path in PROVIDER_DIR.glob("*/")
        if path.is_dir() and not path.stem.startswith("_")
    ]
)

DEFAULT_PROVIDER = PROVIDERS[0]

CONFIG_DIR_NAME = ".cici"



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from importlib import import_module
from pathlib import Path

from termcolor import colored

from ..constants import DEFAULT_PROVIDER


def fmt_command(parser, args):
    provider = import_module(f".{DEFAULT_PROVIDER}", "cici.providers")

    if not args.filenames:
        args.filenames = [provider.CI_FILE]

    for filename in args.filenames:
        if not Path(filename).exists():
            parser.error(f"file not found: {filename}")

    for filename in args.filenames:
        file = provider.load(filename)
        with open(filename, "w") as stream:
            provider.dump(file, stream)
        print(colored("formatted", "magenta"), filename)


def fmt_parser(subparsers):
    parser = subparsers.add_parser("fmt", help="format CI files")
    parser.add_argument("filenames", metavar="FILE", nargs="*")
    parser.set_defaults(func=fmt_command)
    return parser



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import copy
import logging
import re
import sys
from importlib import import_module
from io import StringIO
from pathlib import Path

from termcolor import colored

from ..constants import CONFIG_DIR_NAME, DEFAULT_PROVIDER


def get_bundle_name(text):
    return re.sub(r"[-_ ]+", "-", text.casefold())


def get_bundle_content(provider, bundle):
    content = StringIO()
    content.write(f"#\n")
    content.write(
        f"# This file is generated by cici-tools with the following command:\n"
    )
    content.write(f"#\n")
    content.write(f"#     {' '.join([Path(sys.argv[0]).name, *sys.argv[1:]])}\n")
    content.write(f"#\n")
    provider.dump(bundle, content)
    return content.getvalue()


def bundle_command(parser, args):
    logging.basicConfig(level=logging.INFO, format="[%(levelname)s] %(message)s")

    provider = import_module(f".{DEFAULT_PROVIDER}", "cici.providers")

    ci_file_path = args.config_path / provider.CI_FILE

    if not Path(ci_file_path).exists():
        parser.error(f"file not found: {ci_file_path}")

    file = provider.load(ci_file_path)

    args.output_path = Path(args.output_path)

    for job_name, job in file.jobs.items():
        if job_name.startswith("."):
            continue

        bundle = copy.deepcopy(file)
        bundle.jobs = {}
        bundle.jobs[job_name] = job

        bundle_filename = args.output_path / f"{get_bundle_name(job_name)}.yml"

        content = get_bundle_content(provider, bundle)

        if bundle_filename.exists() and bundle_filename.is_file():
            old_content = open(bundle_filename).read()
            if old_content == content:
                print("  ", colored(bundle_filename.name, "grey"))
                continue

        with open(bundle_filename, "w") as stream:
            stream.write(content)
        print(colored("⚡", "yellow"), bundle_filename.name)


def bundle_parser(subparsers):
    parser = subparsers.add_parser(
        "bundle", help="bundle CI jobs with dependencies into standalone distributions"
    )
    parser.add_argument(
        "config_path",
        metavar="DIR",
        nargs="?",
        type=Path,
        default=(Path.cwd() / CONFIG_DIR_NAME).absolute(),
    )
    parser.add_argument("-g", "--groups", help="job group patterns")
    parser.add_argument(
        "-o",
        "--output",
        metavar="DIR",
        dest="output_path",
        type=Path,
        default=Path.cwd().absolute(),
    )
    parser.set_defaults(func=bundle_command)
    return parser
